using System;
using System.Collections;
using System.Text;
using System.Data;
using System.Data.OleDb;

namespace Telecomunicacao
{
    public interface ITarifario
    {
        /// <summary>
        /// devolve informao sobre um tarifrio atravs do cdigo do tarifrio
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="id"> identificador de um tarifrio vlido</param>
        /// <pre> utilizador valido e id do tarifario valido </pre>
        /// <post> nao altera os dados </post>
        /// <returns> retorna os dados de um tarifario com um determinado id </returns>
        DataSet GetByID(string user, string pwd, long id);

        /// <summary>
        /// devolve informao sobre um tarifrio, atravs do nome do tarifrio
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="name"> nome do tarifrio </param>
        /// <pre> utilizador valido e nome do tarifario valido </pre>
        /// <post> nao altera os dados </post>
        /// <returns> retorna os dados de um tarifario cujo nome  existente </returns>
        DataSet GetByName(string user, string pwd, string name);

        /// <summary>
        /// criar um novo tarifrio
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="name"> nome do novo tarifrio </param>
        /// <param name="comissao"> comisso associada ao tarifrio</param>
        /// <param name="detalhes"> detalhes de tarifas </param>
        /// <param name="id">retorna o identificador do tarifario</param>
        /// <pre>utilizador tem que ser valido e um supervisor</pre>
        /// <post>retorno de um identificador valido</post>
        /// <returns>retorna o status da insero de um novo tarifrio </returns>
        StatusEnum Criar(string user, string pwd, string nome, double comissao, DataSet detalhes, out long id);

        /// <summary>
        /// alterar o tarifario
        /// disponivel so para supervisores
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="id"> identificador do tarifrio vlido </param>
        /// <param name="name"> nome do tarifrio </param>
        /// <param name="comissao"> comisso associada ao tarifrio </param>
        /// <param name="detalhes"> detalhes das tarifas</param>
        /// <pre>utilizador tem que ser valido e um supervisor </pre>
        /// <post>dados do tarifario alterados</post>
        /// <returns> retorna o status de uma alterao dos dados de um tarifrio </returns>
        StatusEnum Alterar(string user, string pwd, long id, string nome, double comissao, DataSet detalhes);

        /// <summary>
        /// criar um dataset vazio para insero de dados
        /// </summary>
        /// <param name="user"> nome do utilizador vlido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <pre> utilizador tem que ser valido </pre>
        /// <post> dataset com esquema, sem dados </post>
        /// <returns> um dataset esquematizado, para insero de dados </returns>
        DataSet CreateDetails(string user, string pwd);

		/// <summary>
		/// Verifica se "todas" as tarifas so vlidas
		/// </summary>
		/// <param name="detalhes"> detalhes de tarifas </param>
		/// <returns>retorna o status da validao das tarifas: StatusEnum.OK ou StatusEnum.NOT_OK</returns>
		//private StatusEnum ValidarTarifas(DataSet detalhes);
	}
    
    internal class TarifarioImpl : ITarifario
    {
        public TarifarioImpl()
        {
            //
            // TODO: Add constructor logic here
            //
        }
        
        /// <summary>
        /// devolve informao sobre um tarifrio atravs do cdigo do tarifrio
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="id"> identificador de um tarifrio vlido</param>
        /// <pre> utilizador valido e id do tarifario valido </pre>
        /// <post> nao altera os dados </post>
        /// <returns> retorna os dados de um tarifario com um determinado id </returns>
        public DataSet GetByID(string user, string pwd, long id)
        {
			DataSet ds = null;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				if (UtilDB.ValidateUser(conn,null,user,pwd,out tipoutil) != StatusEnum.OK)
					return null;

				//
				// criar um DataAdapter para executar o comando e devolver o dataset
				// cabealho dos tarifrios
				//
				//string sSqlCmd = "select * from vwTotalBySale Where CustomerID = ?";
				string sSqlCmd = "SELECT * FROM Tarifario WHERE id = ?";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("id", id);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "Tarifario");

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe (Tarifa)
				//
				//sSqlCmd = "SELECT * FROM vwExpandedSaleDetails WHERE SaleId IN (SELECT SaleID FROM Sales WHERE CustomerId=?)";
				sSqlCmd = "SELECT * FROM Tarifa WHERE tarifario = ?";
				oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("id", id);

				//
				// executar o comando e preencher um Dataset
				//
				int nRows = oAdapter.Fill(ds, "Tarifa");

				//
				// criar relao entre tabelas
				//
				if (nRows != 0)
				{
					// Sem este controle se no existir ligaes daria um erro fatal
					ds.Relations.Add("Detalhes", ds.Tables["Tarifario"].Columns["id"], ds.Tables["Tarifa"].Columns["tarifario"]);
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByID> da classe 'Tarifario': " + ex.Message);
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

        /// <summary>
        /// devolve informao sobre um tarifrio, atravs do nome do tarifrio
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="name"> nome do tarifrio </param>
        /// <pre> utilizador valido e nome do tarifario valido </pre>
        /// <post> nao altera os dados </post>
        /// <returns> retorna os dados de um tarifario cujo nome  existente </returns>
        public DataSet GetByName(string user, string pwd, string name)
        {
			DataSet ds = null;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				if (UtilDB.ValidateUser(conn,null,user,pwd,out tipoutil) != StatusEnum.OK)
					return null;

				//
				// criar um DataAdapter para executar o comando e devolver o dataset
				// cabealho dos tarifrios
				//
				//string sSqlCmd = "select * from vwTotalBySale Where CustomerID = ?";
				string sSqlCmd = "SELECT * FROM Tarifario WHERE nome LIKE ?";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("nome", name+"%");

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "Tarifario");

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe (Tarifa)
				//
				//sSqlCmd = "SELECT * FROM vwExpandedSaleDetails WHERE SaleId IN (SELECT SaleID FROM Sales WHERE CustomerId=?)";
				sSqlCmd = "SELECT * FROM Tarifa WHERE tarifario IN (SELECT id AS tarifario FROM Tarifario WHERE nome LIKE ?)";
				oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("nome", name+"%");

				//
				// executar o comando e preencher um Dataset
				//
				int nRows = oAdapter.Fill(ds, "Tarifa");

				//
				// criar relao entre tabelas
				//
				if (nRows != 0) 
				{
					// Sem este controle se no existir ligaes daria um erro fatal
					ds.Relations.Add("Detalhes", ds.Tables["Tarifario"].Columns["id"], ds.Tables["Tarifa"].Columns["tarifario"]);
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByName> da classe 'Tarifario': " + ex.Message);
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

        /// <summary>
        /// criar um novo tarifrio
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="name"> nome do novo tarifrio </param>
        /// <param name="comissao"> comisso associada ao tarifrio</param>
        /// <param name="detalhes"> detalhes de tarifas </param>
        /// <param name="id">retorna o identificador do tarifario</param>
        /// <pre>utilizador tem que ser valido e um supervisor</pre>
        /// <post>retorno de um identificador valido</post>
        /// <returns>retorna o status da insero de um novo tarifrio </returns>
        public StatusEnum Criar(string user, string pwd, string nome, double comissao, DataSet detalhes, out long id)
        {
            StatusEnum status = StatusEnum.NOT_OK;
			OleDbConnection conn = null;
			OleDbTransaction  tx = null;
			TipoUtilizador tipoutil;
            
			// Todo
			/**/
			id = -1;

			if (nome == null || nome.Trim().Length == 0)
			{
				return status = StatusEnum.INVALID_ARGUMENT;
			}
			
			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, tx, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//
				// validar os detalhes das tarifas
				//
				if (ValidarTarifas(detalhes) != StatusEnum.OK)
				{
					return status = StatusEnum.TARIFA_INVALIDA;
				}

				// ======================================
				//
				// tratar das inseres
				//
				// ======================================

				//
				// criar comando para insero
				//
				OleDbCommand cmd = new OleDbCommand("INSERT INTO tarifario (nome, comissao) VALUES (?, ?)", conn);
				cmd.Transaction = tx;
				cmd.Parameters.Add("nome", nome);
				cmd.Parameters.Add("comissao", comissao);

				//
				// inserir registo: tarifario
				//
				cmd.ExecuteNonQuery();

				//
				// obter id autogerado ( que tambm ser retornado! )
				//
				OleDbCommand cmd2 = new OleDbCommand("SELECT @@IDENTITY", conn);
				cmd2.Transaction = tx;
				id = (int)cmd2.ExecuteScalar();

				//
				// inserir chave estrangeira nas tarifas(linhas de detalhe de tarifario)
				//
				foreach (DataRow regTarifa in detalhes.Tables["Tarifa"].Rows)
				{
					regTarifa["tarifario"] = id;
				}

				//
				// inserir Tarifas(linhas de detalhe de Tarifario)
				//
				OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM tarifa", conn);
				da.SelectCommand.Transaction = tx;
				OleDbCommandBuilder cb = new OleDbCommandBuilder(da);
				da.Update(detalhes, "tarifa");

			    //
				// fechar transaco
				//
				tx.Commit();
			}
			catch(OleDbException ex)
			{
				//
				// descartar mudanas efectuadas
				//
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Criar> da classe 'Tarifario': " + ex.Message);
			}
			finally
			{
				conn.Close();
			}

			/**/

            return status = StatusEnum.OK;
        }

        /// <summary>
        /// alterar o tarifario
        /// disponivel so para supervisores
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="id"> identificador do tarifrio vlido </param>
        /// <param name="name"> nome do tarifrio </param>
        /// <param name="comissao"> comisso associada ao tarifrio </param>
        /// <param name="detalhes"> detalhes das tarifas</param>
        /// <pre>utilizador tem que ser valido e um supervisor </pre>
        /// <post>dados do tarifario alterados</post>
        /// <returns> retorna o status de uma alterao dos dados de um tarifrio </returns>
        public StatusEnum Alterar(string user, string pwd, long id, string nome, double comissao, DataSet detalhes)
        {
			StatusEnum status = StatusEnum.NOT_OK;
			OleDbConnection conn = null;
			OleDbTransaction  tx = null;
			TipoUtilizador tipoutil;
            
			// Todo
			/**/
			id = -1;

			//
			// validar dados de entrada
			//
			if (nome == null || nome.Trim().Length == 0)
			{
				return status = StatusEnum.INVALID_ARGUMENT;
			}
			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, tx, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o tarifario existe
				DataSet ds = UtilDB.GetById(conn, tx, "Tarifario", "id", id);
				if (ds.Tables["Tarifario"].Rows.Count == 0)
				{
					return StatusEnum.TARIFARIO_INVALIDO;
				}

				//
				// validar os detalhes das tarifas
				//
				if (ValidarTarifas(detalhes) != StatusEnum.OK)
				{
					return status = StatusEnum.TARIFA_INVALIDA;
				}

				// ======================================
				//
				// tratar das inseres
				//
				// ======================================

				//
				// criar comando para insero
				//
				OleDbCommand cmd = new OleDbCommand("UPDATE tarifario SET nome=?, comissao=? WHERE id=?", conn);
				cmd.Transaction = tx;
				cmd.Parameters.Add("nome", nome);
				cmd.Parameters.Add("comissao", comissao);
				cmd.Parameters.Add("id", id);
				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// actualizou registo
					status = StatusEnum.NOT_OK;
				else
				{
					// no actualizou registo
					status = StatusEnum.OK;		
				}

				//
				// inserir registo: tarifario
				//
				cmd.ExecuteNonQuery();

				//
				// obter id autogerado ( que tambm ser retornado! )
				//
				OleDbCommand cmd2 = new OleDbCommand("SELECT @@IDENTITY", conn);
				cmd2.Transaction = tx;
				id = (int)cmd2.ExecuteScalar();

				//
				// vamos eliminar as tarifas(linhas de detalhe) obsoletas
				//
				OleDbCommand cmd3 = new OleDbCommand("DELETE * FROM tarifa WHERE tarifario=?", conn);
				cmd3.Transaction = tx;
				cmd3.Parameters.Add("id", id);
			
				//
				// executar o comando 
				//
				cmd3.ExecuteNonQuery();

				//
				// inserir Tarifas(linhas de detalhe de Tarifario)
				//
				OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM tarifa", conn);
				da.SelectCommand.Transaction = tx;
				OleDbCommandBuilder cb = new OleDbCommandBuilder(da);
				da.Update(detalhes, "tarifa");

				//
				// fechar transaco
				//
				tx.Commit();
			}
			catch(OleDbException ex)
			{
				//
				// descartar mudanas efectuadas
				//
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Alterar> da classe 'Tarifario': " + ex.Message);
			}
			finally
			{
				conn.Close();
			}

			/**/

			return status = StatusEnum.OK;
		}

        public DataSet CreateDetails(string user, string pwd)
        {
            OleDbConnection conn = null;
            DataSet ds = null;
            TipoUtilizador tipo;

            try
            {
                //
                // criar objecto de conexo  base de dados e abrir a conexo
                //
                conn = new OleDbConnection(UtilDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                if (UtilDB.ValidateUser(conn, null, user, pwd, out tipo) != StatusEnum.OK)
                    return null;

                //
                // criar o dataset
                //
                ds = UtilDB.GetById(conn, null, "Tarifa", "id", -1); ;
                if (ds == null)
                    return null;

                //
                // configurar as colunas para utilizao
                //
                ds.Tables["Tarifa"].Columns["id"].AllowDBNull = true;
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepo!!!!
                //
                System.Console.WriteLine("EXCEPO no mtodo <CreateDetails> da classe 'Tarifario': " + ex.Message);
                return null;
            }
            finally
            {
                //
                // fechar a conexo
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

		/// <summary>
		/// Verifica se "todas" as tarifas so vlidas
		/// </summary>
		/// <param name="detalhes"> detalhes de tarifas </param>
		/// <returns>retorna o status da validao das tarifas: StatusEnum.OK ou StatusEnum.NOT_OK</returns>
		private StatusEnum ValidarTarifas(DataSet detalhes)
		{
			foreach (DataRow regTarifa in detalhes.Tables["Tarifa"].Rows)
			{
				DiaDaSemana diaInicio= (DiaDaSemana)regTarifa["diaSemanaIni"];
				DiaDaSemana diaFim   = (DiaDaSemana)regTarifa["diaSemanaFim"];
				DateTime hrInicio, hrFim;

				// verificar se os dias da semana so vlidos
				if ((diaInicio < DiaDaSemana.DOMINGO) || 
					(diaInicio > DiaDaSemana.FERIADO) ||
					(diaFim < DiaDaSemana.DOMINGO) || 
					(diaFim > DiaDaSemana.FERIADO))
				{
					return StatusEnum.NOT_OK;
				}

				// Se o dia de incio e o dia de fim so o mesmo, 
				// no permitir hora de fim menor ou igual que hora de inicio
				if (diaInicio == diaFim) 
				{
					hrInicio = Convert.ToDateTime(regTarifa["horaInicio"]);
					hrFim    = Convert.ToDateTime(regTarifa["horaFim"]);
					//TimeSpan hrDiff   = hrFim - hrInicio;
					if (hrFim < hrInicio)
					{
						return StatusEnum.NOT_OK;
					}
				}

				// validar campo segundos
				if ((int)regTarifa["segundos"] < 0) 
				{
					return StatusEnum.NOT_OK;
				}

				// validar custo da chamada
				if ((double)regTarifa["custo"] < 0.0) 
				{
					return StatusEnum.NOT_OK;
				}
			}

			// Se chegou aqui  porque est tudo bem
			return StatusEnum.OK;
		}

	}
}
